﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Text;
using EPiServer.Web.Hosting;
using System.Web.Hosting;
using System.IO;
using EPiServer;

namespace BearWare.PixlrGadget.Content.PixlrGadget
{
    public partial class FileInfoJson : SimplePage
    {
        public string FileName { get; set; }
        public string FileSize { get; set; }
        public int Width { get; set; }
        public int Height { get; set; }
        public int PreviewWidth { get; set; }
        public int PreviewHeight { get; set; }
        public string Error { get; set; }
        public string Format { get; set; }
        public int previewSize { get; set; }

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsSupportedFormat(Request.QueryString["filepath"]))
                Error = Translate("/BearWare/PixlrPlugin/FormatNotSupported");
            else
            {
                LoadSettings(new Guid(Request.QueryString["gadgetid"]));
                GetFileInfo(Request.QueryString["filepath"]);
            }
        }

        public string GeneratedJson
        {
            get
            {
                StringBuilder sb = new StringBuilder();
                sb.Append("{");
                if (Error != null)
                {
                    sb.AppendFormat("Error:\"{0}\"", Error);
                }
                else
                {
                    sb.AppendFormat("\"FileName\":\"{0}\",", FileName);
                    sb.AppendFormat("\"FileSize\":\"{0}\",", FileSize);
                    sb.AppendFormat("\"Width\":\"{0}\",", Width);
                    sb.AppendFormat("\"Height\":\"{0}\",", Height);
                    sb.AppendFormat("\"PreviewWidth\":\"{0}\",", PreviewWidth);
                    sb.AppendFormat("\"PreviewHeight\":\"{0}\"", PreviewHeight);
                }
                sb.Append("}");

                return sb.ToString();
            }
        }

        private bool IsSupportedFormat(string filePath)
        {
            Format = filePath.Substring(filePath.LastIndexOf('.')).ToLower();
            if (Format.EndsWith("jpg") ||
                Format.EndsWith("jpeg") ||
                Format.EndsWith("png") ||
                Format.EndsWith("bmp") ||
                Format.EndsWith("pxd") ||
                Format.EndsWith("gif"))
                return true;
            return false;
        }

        private void LoadSettings(Guid gadgetId)
        {
            previewSize = Data.LoadViewModel(gadgetId).Settings.PreviewSize;
        }

        private void GetFileInfo(string filepath)
        {
            System.Drawing.Image gdiImage;
            VirtualFile file = HostingEnvironment.VirtualPathProvider.GetFile(filepath);

            if (file != null)
            {
                if (Format != "pxd") // can't read pxd
                {
                    try
                    {
                        using (Stream unifiedInputStream = file.Open())
                        {
                            gdiImage = System.Drawing.Image.FromStream(unifiedInputStream);
                            Width = gdiImage.Width;
                            Height = gdiImage.Height;

                            // Set the Max size of the thumb.
                            if ((gdiImage.Height > previewSize) || (gdiImage.Width > previewSize))
                            {
                                double heightRatio = (double)gdiImage.Height / previewSize;
                                double widthRatio = (double)gdiImage.Width / previewSize;
                                double usedRatio = heightRatio > widthRatio ? heightRatio : widthRatio;
                                PreviewHeight = (int)(gdiImage.Height / usedRatio);
                                PreviewWidth = (int)(gdiImage.Width / usedRatio);

                                gdiImage.Dispose();
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        Error = ex.Message;
                    }
                }
                else
                {
                    Width = Height = PreviewWidth = PreviewHeight = 0;
                }
                UnifiedFile uf = HostingEnvironment.VirtualPathProvider.GetFile(filepath) as UnifiedFile;
                FileSize = string.Format("{0:0.00}", uf.Length / 1024d);
                FileName = uf.Name;
            }
        }
    }
}